@@ -16,9 +16,9 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#pragma once
 
-#ifndef DOSBOX_HARDWARE_H
-#define DOSBOX_HARDWARE_H
+#include <stdio.h> // for FILE*, for OpenCaptureFile()
 
 class Section;
 enum OPL_Mode {
@@ -31,6 +31,7 @@ enum OPL_Mode {
 #define CAPTURE_VIDEO	0x10
 #define CAPTURE_MULTITRACK_WAVE 0x20 /* like CAPTURE_WAVE, but one AVI audio track per mixer channel for pro video production */
 #define CAPTURE_RAWIMAGE	0x40
+#define CAPTURE_NETWORK		0x80
 
 extern Bitu CaptureState;
 
@@ -53,5 +54,3 @@ void CAPTURE_AddImage(Bitu width, Bitu height, Bitu bpp, Bitu pitch, Bitu flags,
 void CAPTURE_AddMidi(bool sysex, Bitu len, uint8_t * data);
 void CAPTURE_VideoStart();
 void CAPTURE_VideoStop();
-
-#endif
