@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,11 +16,12 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include <assert.h>
 #include <string.h>
 #include "control.h"
 #include "dosbox.h"
 #include "inout.h"
+#include "logging.h"
 #include "setup.h"
 #include "cpu.h"
 #include "../src/cpu/lazyflags.h"
@@ -28,6 +29,8 @@
 
 //#define ENABLE_PORTLOG
 
+#include <math.h> /* floor */
+
 #include <vector>
 
 extern bool pcibus_enable;
